<?php

class IRutils {

  function runSVD($dim) {
    $command = "svd -d $dim -o SVDres termDocMatrix.txt";
    return shell_exec($command); 
  }

  function readS() {
    $s = array();
    $sfile = file('./SVDres-S', FILE_IGNORE_NEW_LINES);
    foreach ($sfile as $i => $line) {
      if ($i==0) {$dim=$line;}
      else {$s[$i-1]=$line;}
    }
    if (count($s) != $dim) {
      $n=count($s);
      exit("Missmatch dimensions in readS: expected $dim; Got $n");
    }
    return $s;
  }

  function readUt() {
    $Ut = array();
    $Mfile = file('./SVDres-Ut', FILE_IGNORE_NEW_LINES);
    foreach ($Mfile as $i => $line) {
      if ($i==0) {$size=explode(' ', $line);$rows=$size[0]; $cols=$size[1];}
      else {
        $Ut[$i-1] = explode(' ', $line);
	if (count($Ut[$i-1]) != $cols) { exit('cols mismatch in readUt'); }
      }
    }
    return $Ut;
  }

}

class Document {
  var $id; #Document ID
  var $termFreq = array(); #term frequency vector
  var $maxTermFreq; #maximum frequency in term frequency vector
  var $length; #Document length in words
  var $title; #Document title
  var $weights = array();
  var $weightsLSI = array();

  function importWords ($text) {
      $words=preg_split('/[^a-zA-Z\'-]+/', $text, -1, PREG_SPLIT_NO_EMPTY);
      foreach ($words as $w) {
        if ( strlen($w) > 1 ) {
          $this->termFreq[strtolower($w)]++;
	  if ($this->maxTermFreq < $this->termFreq[strtolower($w)]) {
	    $this->maxTermFreq = $this->termFreq[strtolower($w)];
	  }
	  $this->length++;
        }
      }
  }

}

class Collection {
  var $docs = array(); #Documents in collection
  var $noOfDocs; #Total no of documents in collection
  var $termDocFreq = array(); #No of documents with term
  var $dimensionsLSI; #no of dimensions to use in LSI
  var $Sinv = array(); #LSI
  var $Ut = array();   #LSI
  var $wLSI = array(); #LSI
  var $mapDocs = array(); #Map documents to integers
  var $mapDocsInv = array(); #Map integers to documents
  var $mapTerms = array(); #Map terms to integers
  var $mapTermsInv = array(); #Map integers to terms
  private $util; # = new IRutils;
  function __construct() {
     $this->util = new IRutils;
  }

  function addDoc($id,$title,$text) {
    $d = new Document;
    $d->id = $id;
    $d->title = $title;
    $d->importWords($text);
    $this->docs[] = $d;
    $this->noOfDocs++;
    return $d;
  }

  function printFreqMatrix () {
    printf("%12s", "DocId");
    foreach ($this->docs as $d) {
      printf("%6s",substr($d->id,0,5));
    }
    print " DocFreq";
    print "\n";
    foreach ($this->termDocFreq as $word => $docfreq) {
      printf("%12s", $word);
      foreach ($this->docs as $d) {
        printf("%6d",$d->termFreq[$word]);
      }
      printf("%6d",$docfreq); 
      print "\n";
    }
  }


## SVD stuff ##
  function mapDocsTerms2numbers() {
  #Map documentIds and terms to numbers and vice versa
  #Required by the saveWeightMatrixCompressed() function
    $n=0;
    foreach ($this->docs as $d) {
      $this->mapDocs[$d->id]=$n;
      $this->mapDocsInv[$n]=$d->id;
      $n++;
    }

    $n=0;
    foreach ($this->termDocFreq as $word => $docfreq) {
      $this->mapTerms[$word]=$n;
      $this->mapTermsInv[$n]=$word;
      $n++;
    }
  }

  function saveWeightMatrixCompressed() { #for SVD
  #You must call mapDocsTerms2numbers() before using this function
    $wm = array();
    $colNonZero = array();
    $t=count($this->mapTerms); #terms=rows
    $N=count($this->mapDocs); #docs=cols
    $antNonZero=0;
    #document-term weight matrix $t x $N
    foreach ($this->mapTerms as $word => $i) {   #terms=rows
      foreach ($this->docs as $d) { #docs=cols
        $j=$this->mapDocs[$d->id];
        $wm[$i][$j]=$d->weights[$word];
        if ($wm[$i][$j]>0) {
          $antNonZero++;
          $colNonZero[$j]++;
        }
      }
    }
    $res = "$t $N $antNonZero\n";
    foreach ($this->mapDocs as $recid => $j) {
      $n=$colNonZero[$j];
      $res .= "$n\n";
      foreach ($this->mapTerms as $word => $i) {
        $val = $wm[$i][$j];
        if ($val>0) { $res .= "$i $val\n"; }
      }
    }
    file_put_contents('termDocMatrix.txt', $res);
  }

}

?>
